/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items;

import java.util.EnumMap;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.util.Size;
import net.dries007.tfc.api.util.Weight;
import net.dries007.tfc.objects.Gem;
import net.dries007.tfc.util.OreDictionaryHelper;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class ItemGem extends ItemTFC
{
    private static final EnumMap<Gem, ItemGem> MAP = new EnumMap<>(Gem.class);

    public static ItemGem get(Gem gem)
    {
        return MAP.get(gem);
    }

    public static ItemStack get(Gem ore, Gem.Grade grade, int amount)
    {
        return new ItemStack(MAP.get(ore), amount, grade.getMeta());
    }

    public final Gem gem;

    public ItemGem(Gem gem)
    {
        this.gem = gem;
        if (MAP.put(gem, this) != null) throw new IllegalStateException("There can only be one.");
        func_77656_e(0);
        func_77627_a(true);
        OreDictionaryHelper.register(this, "gem");
        OreDictionaryHelper.register(this, "gem", gem);
        for (Gem.Grade grade : Gem.Grade.values())
        {
            OreDictionaryHelper.registerMeta(this, grade.getMeta(), "gem", gem, grade);
            OreDictionaryHelper.registerMeta(this, grade.getMeta(), "gem", grade);
        }
    }

    public Gem.Grade getGradeFromStack(ItemStack stack)
    {
        return Gem.Grade.fromMeta(stack.func_77952_i());
    }

    @Override
    public String func_77667_c(ItemStack stack)
    {
        Gem.Grade grade = getGradeFromStack(stack);
        return super.func_77667_c(stack) + "." + grade.name().toLowerCase();
    }

    @Override
    public void func_150895_a(CreativeTabs tab, NonNullList<ItemStack> items)
    {
        if (!func_194125_a(tab)) return;

        for (Gem.Grade grade : Gem.Grade.values())
            items.add(new ItemStack(this, 1, grade.getMeta()));
    }

    @Override
    public Size getSize(ItemStack stack)
    {
        return Size.TINY;
    }

    @Override
    public Weight getWeight(ItemStack stack)
    {
        return Weight.LIGHT;
    }
}
