/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.world.classic.chunkdata;

import javax.annotation.Nonnull;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;

import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.api.types.Ore;
import net.dries007.tfc.util.OreSpawnData;

public class ChunkDataOreSpawned
{
    public final Ore ore;
    public final OreSpawnData.SpawnSize size;
    public final Ore.Grade grade;
    public final BlockPos pos;

    public ChunkDataOreSpawned(@Nonnull Ore ore, OreSpawnData.SpawnSize size, Ore.Grade grade, BlockPos pos)
    {
        this.ore = ore;
        this.size = size;
        this.grade = grade;
        this.pos = pos;
    }

    public ChunkDataOreSpawned(NBTTagCompound tag)
    {
        ore = TFCRegistries.ORES.getValue(new ResourceLocation(tag.func_74779_i("ore")));
        grade = Ore.Grade.valueOf(tag.func_74779_i("grade").toUpperCase());
        size = OreSpawnData.SpawnSize.valueOf(tag.func_74779_i("size").toUpperCase());
        pos = NBTUtil.func_186861_c(tag.func_74775_l("pos"));
    }

    public NBTTagCompound serialize()
    {
        NBTTagCompound root = new NBTTagCompound();
        root.func_74778_a("ore", ore.getRegistryName().toString());
        root.func_74778_a("grade", grade.name());
        root.func_74778_a("size", size.name());
        root.func_74782_a("pos", NBTUtil.func_186859_a(pos));
        return root;
    }

    @Override
    public String toString()
    {
        return "ChunkDataOreSpawned{" +
            "ore=" + ore +
            ", size=" + size +
            ", grade=" + grade +
            ", pos=" + pos +
            '}';
    }
}
